@@ -1,6 +1,7 @@
 from ._zernpol import (Zernpol, zernpol, ZIS, zernpol_pyramid, latex_formula, zernpol_norm, 
                       zernpol_func, zernpol_func_cart, zernrange, zernpol_to_noll, noll_to_zernpol,
                       zernpol_to_ansi, ansi_to_zernpol, zernpol_to_fringe, fringe_to_zernpol, 
-                      zernpol_to_wyant, wyant_to_zernpol, zernpol_info
+                      zernpol_to_wyant, wyant_to_zernpol, zernpol_info, zernpol_pupil
                       )
-from ._tools import cart2pol, pol2cart
+from ._tools import cart2pol, pol2cart, nanrms
+from ._pupil import PupilDisk, PupilMask, mask_from_image
