@@ -2,6 +2,7 @@ import numpy as np
 from collections import namedtuple
 from typing import Iterable, Tuple, Union, Optional
 from ._tools import cart2pol
+from ._pupil import PupilMask 
 
 # dictionary of zernike polynom informations 
 _zerpol_info = {
@@ -160,7 +161,7 @@ def zernpol_func1(nm, r, theta, normalized=True, masked=True):
         rnm *= np.cos(theta*m)
     elif im<0:
         rnm *= np.sin(theta*m)
-    if masked:
+    if masked:        
         if rnm.shape:
             rnm[r>1.0] = np.nan
         else:
@@ -261,6 +262,65 @@ def zernpol_func_cart(nm, x, y, normalized=True, masked=True):
     z = zernpol_func(nm, r, theta, normalized=normalized, masked=masked)    
     return z
 
+
+_flip_func_loockup = {
+(1,  1) : lambda t: t, 
+(-1, 1) : lambda t: -t + np.pi,
+(-1,-1) : lambda t: t  + np.pi,
+(1, -1) : lambda t: -t,
+}   
+def reorient(theta, flip, offset_angle):
+    try:
+        f = _flip_func_loockup[tuple(flip)]
+    except KeyError:
+        raise ValueError(f"flip argument not understood got: {flip!r}")
+    if offset_angle: # avoid to add 0.0 to a big theta array 
+        return f(theta) + offset_angle
+    else:
+        return f(theta)
+       
+def zernpol_pupil(nma, pupil: PupilMask, inpupil_only=False, normalized: bool=True):
+    """ Zernike Polynomial from a :class:`PupilMask`
+    
+    Args:
+        nma (tuple, array of tuple): define the zernike polynomials coefficient. 
+             The last dimension must be 2  
+        pupil (:class:`PupilMask`): The pupil mask object define the mask array and how the zernike
+              mode will be built inside the pupil image
+        normalized (boolean, optional): if True (default), the polynom is normalized to 1
+     
+    Return:
+        z (array like) : float of array with shape  nma.shape[:-1]+pupil.mask.shape  
+    
+    Example:
+    
+    ::
+    
+        >>> from matplotlib.pylab import plt 
+        >>> from zernpol import PupilDisk, zernpol, zernpol_pupil
+        >>> disk = PupilDisk(100) # 100 diameter in user unit
+        >>> disk_mask = disk.make_mask( (120,120), 1.0, center=(60.,60) )
+        >>> z = zernpol_pupil( zernpol(5), disk_mask)
+        >>> plt.imshow(z)
+        >>> plt.show()
+          
+        
+    """     
+    mask = pupil.mask
+    nY, nX =  mask.shape
+    cx, cy = pupil.center
+    rad = pupil.radius
+    X, Y = np.meshgrid(np.arange(nX), np.arange(nY))
+    
+    r, theta = cart2pol((X-cx)/(rad) , (Y-cy)/(rad))    
+    theta = reorient(theta, pupil.flip, pupil.angle)
+    z =  zernpol_func(nma, r, theta, normalized=normalized, masked=True)
+    if inpupil_only:
+        return z[...,mask]
+    z[...,~mask] = np.nan
+    return z   
+    
+
 class Zernpol(namedtuple('BaseZernpolTuple',['n', 'm'])):
     """ Usefull class holding zernike polynoms coefficient 
     
@@ -333,8 +393,8 @@ class Zernpol(namedtuple('BaseZernpolTuple',['n', 'm'])):
             masked (boolean, optional): if True all values outside the r=1 (above r>) is masked by nan values
          
         
-        Ouputs:
-            z: float of array with shape r.shape  
+        Return:
+            z (array like): float of array with shape r.shape  
         
         Example:
         
@@ -361,7 +421,7 @@ class Zernpol(namedtuple('BaseZernpolTuple',['n', 'm'])):
             >>> [plt.plot(r[0], z) for z in a]
             
         """
-        return zernpol_func1(self, r, theta, normalized=normalized, masked=masked)
+        return zernpol_func(self, r, theta, normalized=normalized, masked=masked)
     
     def func_cart(self, x, y, normalized=True, masked=True):
         """ Zernike Polynomial function cartezian coordinates
@@ -375,7 +435,7 @@ class Zernpol(namedtuple('BaseZernpolTuple',['n', 'm'])):
             masked (boolean, optional): if True all values outside the $r^2= x^2+y^2=1$ is masked by nan values
          
         Return:
-            z: float of array with shape x.shape  
+            z (array like): float of array with shape x.shape  
         
         Example:
         
@@ -390,8 +450,10 @@ class Zernpol(namedtuple('BaseZernpolTuple',['n', 'm'])):
             >>> plt.imshow( z4.func_cart(x,y) )
                 
         """
-        return zernpol_func_cart1(self, x, y, normalized=normalized, masked=masked)
+        return zernpol_func_cart(self, x, y, normalized=normalized, masked=masked)
     
+    def func_pupil(self, pupil: PupilMask, inpupil_only=False,  normalized: bool = True):
+        return zernpol_pupil(self, pupil, inpupil_only=inpupil_only, normalized=normalized)
     
     @property
     def info(self):
@@ -568,11 +630,14 @@ def zernpol(input: Union[int,Tuple[int,int],list],
     return _zernpol_walk(input, system)
 
 def zernrange(*args, system=None):
-    """ Return an iterator on zernike polynomial coefficients 
+    """ Return an list on zernike polynomial coefficients 
     
     Args:
        *arg start,top,step: directly parsed 
     
+    Return:
+       zernikes (list) : list of :class:`zernpol.Zernike` zernike polynoms
+       
     Example:
     
     ::
@@ -589,8 +654,7 @@ def zernrange(*args, system=None):
        
     """
     system = ZIS._get(system)
-    for n in range(*args):
-        yield _zerpol1(n, system)
+    return [_zerpol1(n, system) for n in range(*args)]
 
 # this is used as 
 _noll_pol_triangle = sum(zernpol_pyramid(100) , [])
